@@ -3,7 +3,7 @@
  *
  * Implements cluster information actions for the repmgr command line utility
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "repmgr-client-global.h"
 #include "repmgr-action-cluster.h"
 
-#define SHOW_HEADER_COUNT 7
+#define SHOW_HEADER_COUNT 9
 
 typedef enum
 {
@@ -34,6 +34,8 @@ typedef enum
 	SHOW_STATUS,
 	SHOW_UPSTREAM_NAME,
 	SHOW_LOCATION,
+	SHOW_PRIORITY,
+	SHOW_TIMELINE_ID,
 	SHOW_CONNINFO
 }			ShowHeader;
 
@@ -63,6 +65,7 @@ static void cube_set_node_status(t_node_status_cube **cube, int n, int node_id,
  * CLUSTER SHOW
  *
  * Parameters:
+ *   --compact
  *   --csv
  */
 void
@@ -102,12 +105,25 @@ do_cluster_show(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
+	/* Initialize column headers  */
 	strncpy(headers_show[SHOW_ID].title, _("ID"), MAXLEN);
 	strncpy(headers_show[SHOW_NAME].title, _("Name"), MAXLEN);
 	strncpy(headers_show[SHOW_ROLE].title, _("Role"), MAXLEN);
 	strncpy(headers_show[SHOW_STATUS].title, _("Status"), MAXLEN);
 	strncpy(headers_show[SHOW_UPSTREAM_NAME].title, _("Upstream"), MAXLEN);
 	strncpy(headers_show[SHOW_LOCATION].title, _("Location"), MAXLEN);
+
+	if (runtime_options.compact == true)
+	{
+		strncpy(headers_show[SHOW_PRIORITY].title, _("Prio."), MAXLEN);
+		strncpy(headers_show[SHOW_TIMELINE_ID].title, _("TLI"), MAXLEN);
+	}
+	else
+	{
+		strncpy(headers_show[SHOW_PRIORITY].title, _("Priority"), MAXLEN);
+		strncpy(headers_show[SHOW_TIMELINE_ID].title, _("Timeline"), MAXLEN);
+	}
+
 	strncpy(headers_show[SHOW_CONNINFO].title, _("Connection string"), MAXLEN);
 
 	/*
@@ -117,25 +133,58 @@ do_cluster_show(void)
 
 	for (i = 0; i < SHOW_HEADER_COUNT; i++)
 	{
-		headers_show[i].max_length = strlen(headers_show[i].title);
+		headers_show[i].display = true;
+
+		/* Don't display timeline on pre-9.6 clusters  */
+		if (i == SHOW_TIMELINE_ID)
+		{
+			if (PQserverVersion(conn) < 90600)
+			{
+				headers_show[i].display = false;
+			}
+		}
+
+		/* if --compact provided, don't display conninfo */
+		if (runtime_options.compact == true)
+		{
+			if (i == SHOW_CONNINFO)
+			{
+				headers_show[i].display = false;
+			}
+		}
+
+
+		if (headers_show[i].display == true)
+		{
+			headers_show[i].max_length = strlen(headers_show[i].title);
+		}
 	}
 
+	/*
+	 * TODO: count nodes marked as "? unreachable" and add a hint about
+	 * the other cluster commands for better determining whether
+	 * unreachable.
+	 */
+
 	for (cell = nodes.head; cell; cell = cell->next)
 	{
-		PQExpBufferData details;
+		PQExpBufferData node_status;
+		PQExpBufferData upstream;
+		PQExpBufferData buf;
+
+		cell->node_info->replication_info = palloc0(sizeof(ReplInfo));
+		if (cell->node_info->replication_info == NULL)
+		{
+			log_error(_("unable to allocate memory"));
+			exit(ERR_INTERNAL);
+		}
+
+		init_replication_info(cell->node_info->replication_info);
 
 		cell->node_info->conn = establish_db_connection_quiet(cell->node_info->conninfo);
 
-		if (PQstatus(cell->node_info->conn) == CONNECTION_OK)
+		if (PQstatus(cell->node_info->conn) != CONNECTION_OK)
 		{
-			cell->node_info->node_status = NODE_STATUS_UP;
-			cell->node_info->recovery_type = get_recovery_type(cell->node_info->conn);
-		}
-		else
-		{
-			cell->node_info->node_status = NODE_STATUS_DOWN;
-			cell->node_info->recovery_type = RECTYPE_UNKNOWN;
-
 			connection_error_found = true;
 
 			if (runtime_options.verbose)
@@ -154,199 +203,70 @@ do_cluster_show(void)
 										cell->node_info->node_name, cell->node_info->node_id);
 			}
 		}
-
-		initPQExpBuffer(&details);
-
-		/*
-		 * TODO: count nodes marked as "? unreachable" and add a hint about
-		 * the other cluster commands for better determining whether
-		 * unreachable.
-		 */
-		switch (cell->node_info->type)
+		else
 		{
-			case PRIMARY:
-				{
-					/* node is reachable */
-					if (cell->node_info->node_status == NODE_STATUS_UP)
-					{
-						if (cell->node_info->active == true)
-						{
-							switch (cell->node_info->recovery_type)
-							{
-								case RECTYPE_PRIMARY:
-									appendPQExpBufferStr(&details, "* running");
-									break;
-								case RECTYPE_STANDBY:
-									appendPQExpBufferStr(&details, "! running as standby");
-									item_list_append_format(&warnings,
-															"node \"%s\" (ID: %i) is registered as primary but running as standby",
-															cell->node_info->node_name, cell->node_info->node_id);
-									break;
-								case RECTYPE_UNKNOWN:
-									appendPQExpBufferStr(&details, "! unknown");
-									item_list_append_format(&warnings,
-															"node \"%s\" (ID: %i) has unknown replication status",
-															cell->node_info->node_name, cell->node_info->node_id);
-									break;
-							}
-						}
-						else
-						{
-							if (cell->node_info->recovery_type == RECTYPE_PRIMARY)
-							{
-								appendPQExpBufferStr(&details, "! running");
-								item_list_append_format(&warnings,
-														"node \"%s\" (ID: %i) is running but the repmgr node record is inactive",
-														cell->node_info->node_name, cell->node_info->node_id);
-							}
-							else
-							{
-								appendPQExpBufferStr(&details, "! running as standby");
-								item_list_append_format(&warnings,
-														"node \"%s\" (ID: %i) is registered as an inactive primary but running as standby",
-														cell->node_info->node_name, cell->node_info->node_id);
-							}
-						}
-					}
-					/* node is unreachable */
-					else
-					{
-						/* node is unreachable but marked active */
-						if (cell->node_info->active == true)
-						{
-							appendPQExpBufferStr(&details, "? unreachable");
-							item_list_append_format(&warnings,
-													"node \"%s\" (ID: %i) is registered as an active primary but is unreachable",
-													cell->node_info->node_name, cell->node_info->node_id);
-						}
-						/* node is unreachable and marked as inactive */
-						else
-						{
-							appendPQExpBufferStr(&details, "- failed");
-							error_found = true;
-						}
-					}
-				}
-				break;
-			case STANDBY:
-				{
-					/* node is reachable */
-					if (cell->node_info->node_status == NODE_STATUS_UP)
-					{
-						if (cell->node_info->active == true)
-						{
-							switch (cell->node_info->recovery_type)
-							{
-								case RECTYPE_STANDBY:
-									appendPQExpBufferStr(&details, "  running");
-									break;
-								case RECTYPE_PRIMARY:
-									appendPQExpBufferStr(&details, "! running as primary");
-									item_list_append_format(&warnings,
-															"node \"%s\" (ID: %i) is registered as standby but running as primary",
-															cell->node_info->node_name, cell->node_info->node_id);
-									break;
-								case RECTYPE_UNKNOWN:
-									appendPQExpBufferStr(&details, "! unknown");
-									item_list_append_format(
-															&warnings,
-															"node \"%s\" (ID: %i) has unknown replication status",
-															cell->node_info->node_name, cell->node_info->node_id);
-									break;
-							}
-						}
-						else
-						{
-							if (cell->node_info->recovery_type == RECTYPE_STANDBY)
-							{
-								appendPQExpBufferStr(&details, "! running");
-								item_list_append_format(&warnings,
-														"node \"%s\" (ID: %i) is running but the repmgr node record is inactive",
-														cell->node_info->node_name, cell->node_info->node_id);
-							}
-							else
-							{
-								appendPQExpBufferStr(&details, "! running as primary");
-								item_list_append_format(&warnings,
-														"node \"%s\" (ID: %i) is running as primary but the repmgr node record is inactive",
-														cell->node_info->node_name, cell->node_info->node_id);
-							}
-						}
-					}
-					/* node is unreachable */
-					else
-					{
-						/* node is unreachable but marked active */
-						if (cell->node_info->active == true)
-						{
-							appendPQExpBufferStr(&details, "? unreachable");
-							item_list_append_format(&warnings,
-													"node \"%s\" (ID: %i) is registered as an active standby but is unreachable",
-													cell->node_info->node_name, cell->node_info->node_id);
-						}
-						else
-						{
-							appendPQExpBufferStr(&details, "- failed");
-							error_found = true;
-						}
-					}
-				}
-				break;
-			case WITNESS:
-			case BDR:
-				{
-					/* node is reachable */
-					if (cell->node_info->node_status == NODE_STATUS_UP)
-					{
-						if (cell->node_info->active == true)
-						{
-							appendPQExpBufferStr(&details, "* running");
-						}
-						else
-						{
-							appendPQExpBufferStr(&details, "! running");
-							error_found = true;
-						}
-					}
-					/* node is unreachable */
-					else
-					{
-						if (cell->node_info->active == true)
-						{
-							appendPQExpBufferStr(&details, "? unreachable");
-						}
-						else
-						{
-							appendPQExpBufferStr(&details, "- failed");
-							error_found = true;
-						}
-					}
-				}
-				break;
-			case UNKNOWN:
-				{
-					/* this should never happen */
-					appendPQExpBufferStr(&details, "? unknown node type");
-						error_found = true;
-				}
-				break;
+			/* NOP on pre-9.6 servers */
+			cell->node_info->replication_info->timeline_id = get_node_timeline(cell->node_info->conn);
 		}
 
-		strncpy(cell->node_info->details, details.data, MAXLEN);
-		termPQExpBuffer(&details);
+		initPQExpBuffer(&node_status);
+		initPQExpBuffer(&upstream);
+
+		if (format_node_status(cell->node_info, &node_status, &upstream, &warnings) == true)
+			error_found = true;
+
+		snprintf(cell->node_info->details, sizeof(cell->node_info->details),
+				 "%s", node_status.data);
+		snprintf(cell->node_info->upstream_node_name, sizeof(cell->node_info->upstream_node_name),
+				 "%s", upstream.data);
+
+		termPQExpBuffer(&node_status);
+		termPQExpBuffer(&upstream);
 
 		PQfinish(cell->node_info->conn);
 		cell->node_info->conn = NULL;
 
+		initPQExpBuffer(&buf);
+		appendPQExpBuffer(&buf, "%i", cell->node_info->node_id);
+		headers_show[SHOW_ID].cur_length = strlen(buf.data);
+		termPQExpBuffer(&buf);
+
 		headers_show[SHOW_ROLE].cur_length = strlen(get_node_type_string(cell->node_info->type));
 		headers_show[SHOW_NAME].cur_length = strlen(cell->node_info->node_name);
 		headers_show[SHOW_STATUS].cur_length = strlen(cell->node_info->details);
+
 		headers_show[SHOW_UPSTREAM_NAME].cur_length = strlen(cell->node_info->upstream_node_name);
+
+		initPQExpBuffer(&buf);
+		appendPQExpBuffer(&buf, "%i", cell->node_info->priority);
+		headers_show[SHOW_PRIORITY].cur_length = strlen(buf.data);
+		termPQExpBuffer(&buf);
+
 		headers_show[SHOW_LOCATION].cur_length = strlen(cell->node_info->location);
+
+		if (cell->node_info->replication_info->timeline_id == UNKNOWN_TIMELINE_ID)
+		{
+			/* display "?" */
+			headers_show[SHOW_PRIORITY].cur_length = 1;
+		}
+		else
+		{
+			initPQExpBuffer(&buf);
+			appendPQExpBuffer(&buf, "%i", cell->node_info->replication_info->timeline_id);
+			headers_show[SHOW_PRIORITY].cur_length = strlen(buf.data);
+			termPQExpBuffer(&buf);
+		}
+
 		headers_show[SHOW_CONNINFO].cur_length = strlen(cell->node_info->conninfo);
 
 		for (i = 0; i < SHOW_HEADER_COUNT; i++)
 		{
+			if (runtime_options.compact == true)
+			{
+				if (headers_show[i].display == false)
+					continue;
+			}
+
 			if (headers_show[i].cur_length > headers_show[i].max_length)
 			{
 				headers_show[i].max_length = headers_show[i].cur_length;
@@ -398,7 +318,22 @@ do_cluster_show(void)
 			printf("| %-*s ", headers_show[SHOW_STATUS].max_length, cell->node_info->details);
 			printf("| %-*s ", headers_show[SHOW_UPSTREAM_NAME].max_length, cell->node_info->upstream_node_name);
 			printf("| %-*s ", headers_show[SHOW_LOCATION].max_length, cell->node_info->location);
-			printf("| %-*s\n", headers_show[SHOW_CONNINFO].max_length, cell->node_info->conninfo);
+			printf("| %-*i ", headers_show[SHOW_PRIORITY].max_length, cell->node_info->priority);
+
+			if (headers_show[SHOW_TIMELINE_ID].display == true)
+			{
+				if (cell->node_info->replication_info->timeline_id == UNKNOWN_TIMELINE_ID)
+					printf("| %-*c ", headers_show[SHOW_TIMELINE_ID].max_length, '?');
+				else
+					printf("| %-*i ", headers_show[SHOW_TIMELINE_ID].max_length, (int)cell->node_info->replication_info->timeline_id);
+			}
+
+			if (headers_show[SHOW_CONNINFO].display == true)
+			{
+				printf("| %-*s", headers_show[SHOW_CONNINFO].max_length, cell->node_info->conninfo);
+			}
+
+			puts("");
 		}
 	}
 
@@ -447,6 +382,7 @@ do_cluster_show(void)
  *   --node-[id|name]
  *   --event
  *   --csv
+ *   --compact
  */
 
 void
@@ -492,11 +428,11 @@ do_cluster_event(void)
 	strncpy(headers_event[EV_DETAILS].title, _("Details"), MAXLEN);
 
 	/*
-	 * If --terse or --csv provided, simply omit the "Details" column.
+	 * If --compact or --csv provided, simply omit the "Details" column.
 	 * In --csv mode we'd need to quote/escape the contents "Details" column,
 	 * which is doable but which will remain a TODO for now.
 	 */
-	if (runtime_options.terse == true || runtime_options.output_mode == OM_CSV)
+	if (runtime_options.compact == true || runtime_options.output_mode == OM_CSV)
 		column_count --;
 
 	for (i = 0; i < column_count; i++)
@@ -960,7 +896,9 @@ build_cluster_matrix(t_node_matrix_rec ***matrix_rec_dest, int *name_length, Ite
 		matrix_rec_list[i] = (t_node_matrix_rec *) pg_malloc0(sizeof(t_node_matrix_rec));
 
 		matrix_rec_list[i]->node_id = cell->node_info->node_id;
-		strncpy(matrix_rec_list[i]->node_name, cell->node_info->node_name, MAXLEN);
+		strncpy(matrix_rec_list[i]->node_name,
+				cell->node_info->node_name,
+				sizeof(matrix_rec_list[i]->node_name));
 
 		/*
 		 * Find the maximum length of a node name
@@ -1058,6 +996,7 @@ build_cluster_matrix(t_node_matrix_rec ***matrix_rec_dest, int *name_length, Ite
 		(void) remote_command(host,
 							  runtime_options.remote_user,
 							  command.data,
+							  config_file_options.ssh_options,
 							  &command_output);
 
 		p = command_output.data;
@@ -1174,7 +1113,7 @@ build_cluster_crosscheck(t_node_status_cube ***dest_cube, int *name_length, Item
 
 		cube[h] = (t_node_status_cube *) pg_malloc(sizeof(t_node_status_cube));
 		cube[h]->node_id = cell->node_info->node_id;
-		strncpy(cube[h]->node_name, cell->node_info->node_name, MAXLEN);
+		strncpy(cube[h]->node_name, cell->node_info->node_name, sizeof(cube[h]->node_name));
 
 		/*
 		 * Find the maximum length of a node name
@@ -1196,7 +1135,7 @@ build_cluster_crosscheck(t_node_status_cube ***dest_cube, int *name_length, Item
 			/* we don't need the name here */
 			cube[h]->matrix_list_rec[i]->node_name[0] = '\0';
 
-			cube[h]->matrix_list_rec[i]->node_status_list = (t_node_status_rec **) pg_malloc0(sizeof(t_node_status_rec) * nodes.node_count);
+			cube[h]->matrix_list_rec[i]->node_status_list = (t_node_status_rec **) pg_malloc0(sizeof(t_node_status_rec *) * nodes.node_count);
 
 			j = 0;
 
@@ -1270,6 +1209,7 @@ build_cluster_crosscheck(t_node_status_cube ***dest_cube, int *name_length, Item
 			(void) remote_command(host,
 								  runtime_options.remote_user,
 								  quoted_command.data,
+								  config_file_options.ssh_options,
 								  &command_output);
 
 			free_conninfo_params(&remote_conninfo);
@@ -1492,6 +1432,7 @@ do_cluster_help(void)
 	printf(_("  Configuration file or database connection required.\n"));
 	puts("");
 	printf(_("    --csv                     emit output as CSV (with a subset of fields)\n"));
+	printf(_("    --compact                 display only a subset of fields\n"));
 	puts("");
 
 	printf(_("CLUSTER MATRIX\n"));
@@ -1522,12 +1463,13 @@ do_cluster_help(void)
 	printf(_("    --event                   filter specific event\n"));
 	printf(_("    --node-id                 restrict entries to node with this ID\n"));
 	printf(_("    --node-name               restrict entries to node with this name\n"));
+	printf(_("    --compact                 omit \"Details\" column"));
 	printf(_("    --csv                     emit output as CSV\n"));
 	puts("");
 
 	printf(_("CLUSTER CLEANUP\n"));
 	puts("");
-	printf(_("  \"cluster cleanup\" purges records from the \"repmgr.monitor\" table.\n"));
+	printf(_("  \"cluster cleanup\" purges records from the \"repmgr.monitoring_history\" table.\n"));
 	puts("");
 	printf(_("    -k, --keep-history=VALUE  retain indicated number of days of history (default: 0)\n"));
 	puts("");
