@@ -1,6 +1,6 @@
 /*
  * repmgr-action-daemon.h
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 extern void do_daemon_status(void);
 extern void do_daemon_pause(void);
 extern void do_daemon_unpause(void);
+extern void do_daemon_start(void);
+extern void do_daemon_stop(void);
 
 extern void do_daemon_help(void);
 #endif
