@@ -3,7 +3,7 @@
  *
  * Implements actions available for any kind of node
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,7 @@ static CheckStatus do_node_check_replication_lag(PGconn *conn, OutputMode mode,
 static CheckStatus do_node_check_role(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 static CheckStatus do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
+static CheckStatus do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output);
 
 /*
  * NODE STATUS
@@ -66,7 +67,6 @@ do_node_status(void)
 	PGconn	   *conn = NULL;
 
 	t_node_info node_info = T_NODE_INFO_INITIALIZER;
-	char		server_version[MAXLEN];
 	char		cluster_size[MAXLEN];
 	PQExpBufferData output;
 
@@ -76,21 +76,29 @@ do_node_status(void)
 
 	ItemList	warnings = {NULL, NULL};
 	RecoveryType recovery_type = RECTYPE_UNKNOWN;
-	ReplInfo	replication_info = T_REPLINFO_INTIALIZER;
+	ReplInfo	replication_info;
 	t_recovery_conf recovery_conf = T_RECOVERY_CONF_INITIALIZER;
 
 	char		data_dir[MAXPGPATH] = "";
+	int			server_version_num = UNKNOWN_SERVER_VERSION_NUM;
+	char		server_version_str[MAXVERSIONSTR] = "";
 
+	/*
+	 * A database connection is *not* required for this check
+	 */
 	if (runtime_options.is_shutdown_cleanly == true)
 	{
 		return _do_node_status_is_shutdown_cleanly();
 	}
 
+	init_replication_info(&replication_info);
+
+
 	/* config file required, so we should have "conninfo" and "data_directory" */
 	conn = establish_db_connection(config_file_options.conninfo, true);
 	strncpy(data_dir, config_file_options.data_directory, MAXPGPATH);
 
-	server_version_num = get_server_version(conn, NULL);
+	server_version_num = get_server_version(conn, server_version_str);
 
 	/* check node exists  */
 
@@ -101,18 +109,16 @@ do_node_status(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
-	(void) get_server_version(conn, server_version);
-
 	if (get_cluster_size(conn, cluster_size) == false)
 		strncpy(cluster_size, _("unknown"), MAXLEN);
 
 	recovery_type = get_recovery_type(conn);
 
-	get_node_replication_stats(conn, server_version_num, &node_info);
+	get_node_replication_stats(conn, &node_info);
 
 	key_value_list_set(&node_status,
 					   "PostgreSQL version",
-					   server_version);
+					   server_version_str);
 
 	key_value_list_set(&node_status,
 					   "Total data size",
@@ -219,19 +225,27 @@ do_node_status(void)
 
 		ready_files = get_ready_archive_files(conn, data_dir);
 
-		if (runtime_options.output_mode == OM_CSV)
+		if (ready_files == ARCHIVE_STATUS_DIR_ERROR)
 		{
-			key_value_list_set_format(&node_status,
-									  "WALs pending archiving",
-									  "%i",
-									  ready_files);
+			item_list_append_format(&warnings,
+									"- unable to check archive_status directory\n");
 		}
 		else
 		{
-			key_value_list_set_format(&node_status,
-									  "WALs pending archiving",
-									  "%i pending files",
-									  ready_files);
+			if (runtime_options.output_mode == OM_CSV)
+			{
+				key_value_list_set_format(&node_status,
+										  "WALs pending archiving",
+										  "%i",
+										  ready_files);
+			}
+			else
+			{
+				key_value_list_set_format(&node_status,
+										  "WALs pending archiving",
+										  "%i pending files",
+										  ready_files);
+			}
 		}
 
 		if (guc_set(conn, "archive_mode", "=", "off"))
@@ -280,7 +294,7 @@ do_node_status(void)
 				continue;
 			}
 
-			if (is_downstream_node_attached(conn, node_cell->node_info->node_name) == false)
+			if (is_downstream_node_attached(conn, node_cell->node_info->node_name) != NODE_ATTACHED)
 			{
 				missing_nodes_count++;
 				item_list_append_format(&missing_nodes,
@@ -354,7 +368,7 @@ do_node_status(void)
 		initPQExpBuffer(&slotinfo);
 
 		appendPQExpBuffer(&slotinfo,
-						  "%i (of maximal %i; %i missing)",
+						  "%i physical (of maximal %i; %i missing)",
 						  node_info.active_replication_slots + node_info.inactive_replication_slots,
 						  node_info.max_replication_slots,
 						  missing_slots.node_count);
@@ -371,13 +385,13 @@ do_node_status(void)
 							  node_info.inactive_replication_slots);
 
 			item_list_append_format(&warnings,
-									_("- node has %i inactive replication slots"),
+									_("- node has %i inactive physical replication slots"),
 									node_info.inactive_replication_slots);
 
 			for (cell = inactive_replication_slots.head; cell; cell = cell->next)
 			{
 				item_list_append_format(&warnings,
-										"  - %s (%s)", cell->key, cell->value);
+										"  - %s", cell->key);
 			}
 
 			key_value_list_free(&inactive_replication_slots);
@@ -399,7 +413,7 @@ do_node_status(void)
 								  node_info.upstream_node_name,
 								  node_info.upstream_node_id);
 
-		get_replication_info(conn, &replication_info);
+		get_replication_info(conn, node_info.type, &replication_info);
 
 		key_value_list_set_format(&node_status,
 								  "Replication lag",
@@ -654,27 +668,17 @@ _do_node_status_is_shutdown_cleanly(void)
 		node_status = NODE_STATUS_DOWN;
 	}
 
-	log_verbose(LOG_DEBUG, "node status determined as: %s", print_node_status(node_status));
+	log_verbose(LOG_DEBUG, "node status determined as: %s",
+				print_node_status(node_status));
 
-	switch (node_status)
+	appendPQExpBuffer(&output,
+					  "%s", print_node_status(node_status));
+
+	if (node_status == NODE_STATUS_DOWN)
 	{
-		case NODE_STATUS_UP:
-			appendPQExpBufferStr(&output, "RUNNING");
-			break;
-		case NODE_STATUS_SHUTTING_DOWN:
-			appendPQExpBufferStr(&output, "SHUTTING_DOWN");
-			break;
-		case NODE_STATUS_DOWN:
-			appendPQExpBuffer(&output,
-							  "SHUTDOWN --last-checkpoint-lsn=%X/%X",
-							  format_lsn(checkPoint));
-			break;
-		case NODE_STATUS_UNCLEAN_SHUTDOWN:
-			appendPQExpBufferStr(&output, "UNCLEAN_SHUTDOWN");
-			break;
-		case NODE_STATUS_UNKNOWN:
-			appendPQExpBufferStr(&output, "UNKNOWN");
-			break;
+		appendPQExpBuffer(&output,
+						  " --last-checkpoint-lsn=%X/%X",
+						  format_lsn(checkPoint));
 	}
 
 	printf("%s\n", output.data);
@@ -725,10 +729,8 @@ do_node_check(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
-	server_version_num = get_server_version(conn, NULL);
-
 	/* add replication statistics to node record */
-	get_node_replication_stats(conn, server_version_num, &node_info);
+	get_node_replication_stats(conn, &node_info);
 
 	/*
 	 * handle specific checks ======================
@@ -792,6 +794,16 @@ do_node_check(void)
 		exit(return_code);
 	}
 
+	if (runtime_options.data_directory_config == true)
+	{
+		return_code = do_node_check_data_directory(conn,
+												   runtime_options.output_mode,
+												   &node_info,
+												   NULL);
+		PQfinish(conn);
+		exit(return_code);
+	}
+
 
 	if (runtime_options.output_mode == OM_NAGIOS)
 	{
@@ -824,6 +836,9 @@ do_node_check(void)
 	if (do_node_check_missing_slots(conn, runtime_options.output_mode, &node_info, &status_list) != CHECK_STATUS_OK)
 		issue_detected = true;
 
+	if (do_node_check_data_directory(conn, runtime_options.output_mode, &node_info, &status_list) != CHECK_STATUS_OK)
+		issue_detected = true;
+
 	if (runtime_options.output_mode == OM_CSV)
 	{
 		appendPQExpBuffer(&output,
@@ -1151,7 +1166,7 @@ do_node_check_downstream(PGconn *conn, OutputMode mode, CheckStatusList *list_ou
 			continue;
 		}
 
-		if (is_downstream_node_attached(conn, cell->node_info->node_name) == false)
+		if (is_downstream_node_attached(conn, cell->node_info->node_name) != NODE_ATTACHED)
 		{
 			missing_nodes_count++;
 			item_list_append_format(&missing_nodes,
@@ -1393,7 +1408,7 @@ do_node_check_replication_lag(PGconn *conn, OutputMode mode, t_node_info *node_i
 					break;
 			}
 		}
-		else if (lag_seconds < 0)
+		else if (lag_seconds == UNKNOWN_REPLICATION_LAG)
 		{
 			status = CHECK_STATUS_UNKNOWN;
 
@@ -1445,11 +1460,9 @@ do_node_check_replication_lag(PGconn *conn, OutputMode mode, t_node_info *node_i
 	switch (mode)
 	{
 		case OM_OPTFORMAT:
-			{
-				printf("--status=%s %s\n",
-					   output_check_status(status),
-					   details.data);
-			}
+			printf("--status=%s %s\n",
+				   output_check_status(status),
+				   details.data);
 			break;
 		case OM_NAGIOS:
 			printf("REPMGR_REPLICATION_LAG %s: %s\n",
@@ -1618,7 +1631,7 @@ do_node_check_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, Check
 
 	initPQExpBuffer(&details);
 
-	if (server_version_num < 90400)
+	if (PQserverVersion(conn) < 90400)
 	{
 		appendPQExpBufferStr(&details,
 							 _("replication slots not available for this PostgreSQL version"));
@@ -1626,12 +1639,12 @@ do_node_check_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, Check
 	else if (node_info->total_replication_slots == 0)
 	{
 		appendPQExpBufferStr(&details,
-							 _("node has no replication slots"));
+							 _("node has no physical replication slots"));
 	}
 	else if (node_info->inactive_replication_slots == 0)
 	{
 		appendPQExpBuffer(&details,
-						  _("%i of %i replication slots are active"),
+						  _("%i of %i physical replication slots are active"),
 						  node_info->total_replication_slots,
 						  node_info->total_replication_slots);
 	}
@@ -1640,7 +1653,7 @@ do_node_check_slots(PGconn *conn, OutputMode mode, t_node_info *node_info, Check
 		status = CHECK_STATUS_CRITICAL;
 
 		appendPQExpBuffer(&details,
-						  _("%i of %i replication slots are inactive"),
+						  _("%i of %i physical replication slots are inactive"),
 						  node_info->inactive_replication_slots,
 						  node_info->total_replication_slots);
 	}
@@ -1694,7 +1707,7 @@ do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_inf
 
 	initPQExpBuffer(&details);
 
-	if (server_version_num < 90400)
+	if (PQserverVersion(conn) < 90400)
 	{
 		appendPQExpBufferStr(&details,
 							 _("replication slots not available for this PostgreSQL version"));
@@ -1708,7 +1721,7 @@ do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_inf
 		if (missing_slots.node_count == 0)
 		{
 			appendPQExpBufferStr(&details,
-								 _("node has no missing replication slots"));
+								 _("node has no missing physical replication slots"));
 		}
 		else
 		{
@@ -1718,7 +1731,7 @@ do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_inf
 			status = CHECK_STATUS_CRITICAL;
 
 			appendPQExpBuffer(&details,
-							  _("%i replication slots are missing"),
+							  _("%i physical replication slots are missing"),
 							  missing_slots.node_count);
 
 			if (missing_slots.node_count)
@@ -1779,7 +1792,7 @@ do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_inf
 			if (list_output != NULL)
 			{
 				check_status_list_set(list_output,
-									  "Missing replication slots",
+									  "Missing physical replication slots",
 									  status,
 									  details.data);
 			}
@@ -1800,6 +1813,135 @@ do_node_check_missing_slots(PGconn *conn, OutputMode mode, t_node_info *node_inf
 }
 
 
+CheckStatus
+do_node_check_data_directory(PGconn *conn, OutputMode mode, t_node_info *node_info, CheckStatusList *list_output)
+{
+	CheckStatus status = CHECK_STATUS_OK;
+	char actual_data_directory[MAXPGPATH] = "";
+	PQExpBufferData details;
+
+	if (mode == OM_CSV && list_output == NULL)
+	{
+		log_error(_("--csv output not provided with --data-directory-config option"));
+		PQfinish(conn);
+		exit(ERR_BAD_CONFIG);
+	}
+
+	initPQExpBuffer(&details);
+	/*
+	 * Check actual data directory matches that in repmgr.conf; note this requires
+	 * a superuser connection
+	 */
+
+	if (is_superuser_connection(conn, NULL) == true)
+	{
+		/* we expect to have a database connection */
+		if (get_pg_setting(conn, "data_directory", actual_data_directory) == false)
+		{
+			appendPQExpBuffer(&details,
+							  _("unable to determine current \"data_directory\""));
+			status = CHECK_STATUS_UNKNOWN;
+		}
+
+		if (strncmp(actual_data_directory, config_file_options.data_directory, MAXPGPATH) != 0)
+		{
+			if (mode != OM_NAGIOS)
+			{
+				appendPQExpBuffer(&details,
+								  _("configured \"data_directory\" is \"%s\"; "),
+								  config_file_options.data_directory);
+			}
+
+			appendPQExpBuffer(&details,
+							  "actual data directory is \"%s\"",
+							  actual_data_directory);
+
+			status = CHECK_STATUS_CRITICAL;
+		}
+		else
+		{
+			appendPQExpBuffer(&details,
+							  _("configured \"data_directory\" is \"%s\""),
+							  config_file_options.data_directory);
+		}
+	}
+	/*
+	 * If no superuser connection available, sanity-check that the configuration directory looks
+	 * like a PostgreSQL directory and hope it's the right one.
+	 */
+	else
+	{
+		if (mode == OM_TEXT)
+		{
+			log_info(_("connection is not a superuser connection, falling back to simple check"));
+
+			/* XXX add -S/--superuser option */
+			if (PQserverVersion(conn) >= 100000)
+			{
+				log_hint(_("add the \"%s\" user to group \"pg_read_all_settings\""),
+						   PQuser(conn));
+			}
+		}
+
+		if (is_pg_dir(config_file_options.data_directory) == false)
+		{
+			if (mode == OM_NAGIOS)
+			{
+				appendPQExpBufferStr(&details,
+								  _("configured \"data_directory\" is not a PostgreSQL data directory"));
+			}
+			else
+			{
+				appendPQExpBuffer(&details,
+								  _("configured \"data_directory\" \"%s\" is not a PostgreSQL data directory"),
+								  actual_data_directory);
+			}
+
+			status = CHECK_STATUS_CRITICAL;
+		}
+	}
+
+	switch (mode)
+	{
+		case OM_OPTFORMAT:
+			printf("--configured-data-directory=%s\n",
+				   output_check_status(status));
+			break;
+		case OM_NAGIOS:
+			printf("REPMGR_DATA_DIRECTORY %s: %s",
+				   output_check_status(status),
+				   config_file_options.data_directory);
+
+			if (status == CHECK_STATUS_CRITICAL)
+			{
+				printf(" | %s", details.data);
+			}
+			puts("");
+			break;
+		case OM_CSV:
+		case OM_TEXT:
+			if (list_output != NULL)
+			{
+				check_status_list_set(list_output,
+									  "Configured data directory",
+									  status,
+									  details.data);
+			}
+			else
+			{
+				printf("%s (%s)\n",
+					   output_check_status(status),
+					   details.data);
+			}
+		default:
+			break;
+	}
+
+	termPQExpBuffer(&details);
+
+	return status;
+}
+
 
 void
 do_node_service(void)
@@ -1993,7 +2135,9 @@ void
 do_node_rejoin(void)
 {
 	PGconn	   *upstream_conn = NULL;
-	RecoveryType upstream_recovery_type = RECTYPE_UNKNOWN;
+	RecoveryType primary_recovery_type = RECTYPE_UNKNOWN;
+	PGconn	   *primary_conn = NULL;
+
 	DBState		db_state;
 	PGPing		status;
 	bool		is_shutdown = true;
@@ -2005,11 +2149,9 @@ do_node_rejoin(void)
 	t_node_info primary_node_record = T_NODE_INFO_INITIALIZER;
 
 	bool		success = true;
-	int			server_version_num = UNKNOWN_SERVER_VERSION_NUM;
 	int			follow_error_code = SUCCESS;
 
 	/* check node is not actually running */
-
 	status = PQping(config_file_options.conninfo);
 
 	switch (status)
@@ -2035,7 +2177,7 @@ do_node_rejoin(void)
 		log_error(_("database is still running in state \"%s\""),
 				  describe_db_state(db_state));
 		log_hint(_("\"repmgr node rejoin\" cannot be executed on a running node"));
-		exit(ERR_BAD_CONFIG);
+		exit(ERR_REJOIN_FAIL);
 	}
 
 	/* check if cleanly shut down */
@@ -2054,18 +2196,15 @@ do_node_rejoin(void)
 				log_detail(_("pg_rewind will not be able to run"));
 			}
 			log_hint(_("database should be restarted then shut down cleanly after crash recovery completes"));
-			exit(ERR_BAD_CONFIG);
+			exit(ERR_REJOIN_FAIL);
 		}
 	}
 
-
 	/* check provided upstream connection */
 	upstream_conn = establish_db_connection_by_params(&source_conninfo, true);
 
 	/* sanity checks for 9.3 */
-	server_version_num = get_server_version(upstream_conn, NULL);
-
-	if (server_version_num < 90400)
+	if (PQserverVersion(upstream_conn) < 90400)
 		check_93_config();
 
 	if (get_primary_node_record(upstream_conn, &primary_node_record) == false)
@@ -2076,40 +2215,85 @@ do_node_rejoin(void)
 		exit(ERR_BAD_CONFIG);
 	}
 
-	PQfinish(upstream_conn);
-
 	/* connect to registered primary and check it's not in recovery */
-	upstream_conn = establish_db_connection(primary_node_record.conninfo, false);
+	primary_conn = establish_db_connection(primary_node_record.conninfo, false);
 
-	if (PQstatus(upstream_conn) != CONNECTION_OK)
+	if (PQstatus(primary_conn) != CONNECTION_OK)
 	{
-		log_error(_("unable to connect to current primary \"%s\" (node ID: %i)"),
+		RecoveryType upstream_recovery_type = get_recovery_type(upstream_conn);
+
+		log_error(_("unable to connect to current registered primary \"%s\" (ID: %i)"),
 				  primary_node_record.node_name,
 				  primary_node_record.node_id);
-		log_detail(_("primay node conninfo is: \"%s\""),
+		log_detail(_("registered primary node conninfo is: \"%s\""),
 				   primary_node_record.conninfo);
+		/*
+		 * Catch case where provided upstream is not in recovery, but is also
+		 * not registered as primary
+		 */
+
+		if (upstream_recovery_type == RECTYPE_PRIMARY)
+		{
+			log_warning(_("provided upstream connection string is for a server which is not in recovery, but not registered as primary"));
+			log_hint(_("fix repmgr metadata configuration before continuing"));
+		}
+
+		PQfinish(upstream_conn);
 		exit(ERR_BAD_CONFIG);
 	}
 
-	upstream_recovery_type = get_recovery_type(upstream_conn);
+	PQfinish(upstream_conn);
 
-	if (upstream_recovery_type != RECTYPE_PRIMARY)
+	primary_recovery_type = get_recovery_type(primary_conn);
+
+	if (primary_recovery_type != RECTYPE_PRIMARY)
 	{
-		log_error(_("primary server is registered node \"%s\" (ID: %i), but server is not a primary"),
+		log_error(_("primary server is registered as node \"%s\" (ID: %i), but server is not a primary"),
 				  primary_node_record.node_name,
 				  primary_node_record.node_id);
 		/* TODO: hint about checking cluster */
-		PQfinish(upstream_conn);
+		PQfinish(primary_conn);
 
 		exit(ERR_BAD_CONFIG);
 	}
 
+	/*
+	 * sanity-check that it will actually be possible to stream from the new upstream
+	 */
+	{
+		bool can_follow;
+		TimeLineID tli = get_min_recovery_end_timeline(config_file_options.data_directory);
+		XLogRecPtr min_recovery_location = get_min_recovery_location(config_file_options.data_directory);
+
+		/*
+		 * It's possible this was a former primary, so the minRecoveryPoint*
+		 * fields may be empty.
+		 */
+
+		if (min_recovery_location == InvalidXLogRecPtr)
+			min_recovery_location = get_latest_checkpoint_location(config_file_options.data_directory);
+		if (tli == 0)
+			tli = get_timeline(config_file_options.data_directory);
+
+		can_follow = check_node_can_attach(tli,
+										   min_recovery_location,
+										   primary_conn,
+										   &primary_node_record,
+										   true);
+
+		if (can_follow == false)
+		{
+			PQfinish(primary_conn);
+			exit(ERR_REJOIN_FAIL);
+		}
+	}
+
+
 	/*
 	 * --force-rewind specified - check prerequisites, and attempt to execute
   	 * (if --dry-run provided, just output the command which would be executed)
 	 */
 
-
 	if (runtime_options.force_rewind_used == true)
 	{
 		PQExpBufferData msg;
@@ -2122,12 +2306,12 @@ do_node_rejoin(void)
 
 		initPQExpBuffer(&msg);
 
-		if (can_use_pg_rewind(upstream_conn, config_file_options.data_directory, &msg) == false)
+		if (can_use_pg_rewind(primary_conn, config_file_options.data_directory, &msg) == false)
 		{
 			log_error(_("--force-rewind specified but pg_rewind cannot be used"));
 			log_detail("%s", msg.data);
 			termPQExpBuffer(&msg);
-			PQfinish(upstream_conn);
+			PQfinish(primary_conn);
 
 			exit(ERR_BAD_CONFIG);
 		}
@@ -2186,8 +2370,8 @@ do_node_rejoin(void)
 		else
 		{
 			log_notice(_("executing pg_rewind"));
-			log_debug("pg_rewind command is:\n  %s",
-					  command.data);
+			log_detail(_("pg_rewind command is \"%s\""),
+					   command.data);
 
 			initPQExpBuffer(&command_output);
 
@@ -2203,7 +2387,7 @@ do_node_rejoin(void)
 
 				termPQExpBuffer(&command_output);
 
-				exit(ERR_BAD_CONFIG);
+				exit(ERR_REJOIN_FAIL);
 			}
 
 			termPQExpBuffer(&command_output);
@@ -2292,6 +2476,8 @@ do_node_rejoin(void)
 
 						termPQExpBuffer(&slotdir_ent_path);
 					}
+
+					closedir(slotdir);
 				}
 				termPQExpBuffer(&slotdir_path);
 			}
@@ -2306,26 +2492,34 @@ do_node_rejoin(void)
 
 	initPQExpBuffer(&follow_output);
 
-	success = do_standby_follow_internal(upstream_conn,
+	/*
+	 * do_standby_follow_internal() can handle situations where the follow
+	 * target is not the primary, so requires database handles to both
+	 * (even if they point to the same node). For the time being,
+	 * "node rejoin" will only attach a standby to the primary.
+	 */
+	success = do_standby_follow_internal(primary_conn,
+										 primary_conn,
 										 &primary_node_record,
 										 &follow_output,
+										 ERR_REJOIN_FAIL,
 										 &follow_error_code);
 
 	if (success == false)
 	{
-		log_notice(_("NODE REJOIN failed"));
+		log_error(_("NODE REJOIN failed"));
 
 		if (strlen(follow_output.data))
 			log_detail("%s", follow_output.data);
 
-		create_event_notification(upstream_conn,
+		create_event_notification(primary_conn,
 								  &config_file_options,
 								  config_file_options.node_id,
 								  "node_rejoin",
 								  success,
 								  follow_output.data);
 
-		PQfinish(upstream_conn);
+		PQfinish(primary_conn);
 
 		termPQExpBuffer(&follow_output);
 		exit(follow_error_code);
@@ -2368,9 +2562,10 @@ do_node_rejoin(void)
 
 		for (;  i < config_file_options.node_rejoin_timeout; i++)
 		{
-			success = is_downstream_node_attached(upstream_conn, config_file_options.node_name);
+			NodeAttached node_attached = is_downstream_node_attached(primary_conn,
+														config_file_options.node_name);
 
-			if (success == true)
+			if (node_attached == NODE_ATTACHED)
 			{
 				log_verbose(LOG_INFO, _("node %i has attached to its upstream node"),
 							config_file_options.node_id);
@@ -2379,9 +2574,13 @@ do_node_rejoin(void)
 
 			if (i % 5 == 0)
 			{
-				log_info(_("waiting for node %i to connect to new primary; %i of max %i attempts"),
+				log_info(_("waiting for node \"%s\" (ID: %i) to connect to new primary; %i of max %i attempts"),
+						 config_file_options.node_name,
 						 config_file_options.node_id,
 						 i + 1, config_file_options.node_rejoin_timeout);
+				log_detail(_("checking for record in node \"%s\"'s \"pg_stat_replication\" table where \"application_name\" is \"%s\""),
+						   primary_node_record.node_name,
+						   config_file_options.node_name);
 			}
 			else
 			{
@@ -2393,7 +2592,7 @@ do_node_rejoin(void)
 			sleep(1);
 		}
 
-		create_event_notification(upstream_conn,
+		create_event_notification(primary_conn,
 								  &config_file_options,
 								  config_file_options.node_id,
 								  "node_rejoin",
@@ -2403,13 +2602,20 @@ do_node_rejoin(void)
 		if (success == false)
 		{
 			termPQExpBuffer(&follow_output);
-			log_notice(_("NODE REJOIN failed"));
+			log_error(_("NODE REJOIN failed"));
+			log_detail(_("no record for local node \"%s\" found in node \"%s\"'s \"pg_stat_replication\" table"),
+					   config_file_options.node_name,
+					   primary_node_record.node_name);
+			log_hint(_("check the PostgreSQL log on the local node"));
 			exit(ERR_REJOIN_FAIL);
 		}
 	}
 	else
 	{
-		success = is_downstream_node_attached(upstream_conn, config_file_options.node_name);
+		/* -W/--no-wait provided - check once */
+		NodeAttached node_attached = is_downstream_node_attached(primary_conn, config_file_options.node_name);
+		if (node_attached == NODE_ATTACHED)
+			success = true;
 	}
 
 	/*
@@ -2480,6 +2686,48 @@ do_node_rejoin(void)
 }
 
 
+/*
+ * Currently for testing purposes only, not documented;
+ * use at own risk!
+ */
+
+void
+do_node_control(void)
+{
+	PGconn	   *conn = NULL;
+	pid_t	    wal_receiver_pid = UNKNOWN_PID;
+	conn = establish_db_connection(config_file_options.conninfo, true);
+
+	if (runtime_options.disable_wal_receiver == true)
+	{
+		wal_receiver_pid = disable_wal_receiver(conn);
+
+		PQfinish(conn);
+
+		if (wal_receiver_pid == UNKNOWN_PID)
+			exit(ERR_BAD_CONFIG);
+
+		exit(SUCCESS);
+	}
+
+	if (runtime_options.enable_wal_receiver == true)
+	{
+		wal_receiver_pid = enable_wal_receiver(conn, true);
+
+		PQfinish(conn);
+
+		if (wal_receiver_pid == UNKNOWN_PID)
+			exit(ERR_BAD_CONFIG);
+
+		exit(SUCCESS);
+	}
+
+	log_error(_("no option provided"));
+
+	PQfinish(conn);
+}
+
+
 /*
  * For "internal" use by `node rejoin` on the local node when
  * called by "standby switchover" from the remote node.
@@ -2541,6 +2789,7 @@ _do_node_archive_config(void)
 
 	arcdir = opendir(archive_dir.data);
 
+	/* always attempt to open the directory */
 	if (arcdir == NULL)
 	{
 		log_error(_("unable to open archive directory \"%s\""),
@@ -2586,10 +2835,11 @@ _do_node_archive_config(void)
 
 			termPQExpBuffer(&arcdir_ent_path);
 		}
-
-		closedir(arcdir);
 	}
 
+	closedir(arcdir);
+
+
 	/*
 	 * extract list of config files from --config-files
 	 */
@@ -2607,7 +2857,7 @@ _do_node_archive_config(void)
 			{
 				int			filename_len = j - i;
 
-				if (filename_len > MAXPGPATH)
+				if (filename_len >= MAXPGPATH)
 					filename_len = MAXPGPATH - 1;
 
 				strncpy(filenamebuf, runtime_options.config_files + i, filename_len);
@@ -2861,11 +3111,12 @@ copy_file(const char *src_file, const char *dest_file)
 	int			a = 0;
 
 	ptr_old = fopen(src_file, "r");
-	ptr_new = fopen(dest_file, "w");
 
 	if (ptr_old == NULL)
 		return false;
 
+	ptr_new = fopen(dest_file, "w");
+
 	if (ptr_new == NULL)
 	{
 		fclose(ptr_old);
@@ -2922,8 +3173,8 @@ do_node_help(void)
 	puts("");
 	printf(_("  Configuration file required, runs on local node only.\n"));
 	puts("");
-	printf(_("    --csv                   emit output as CSV\n"));
-	printf(_("    --nagios                emit output in Nagios format (individual status output only)\n"));
+	printf(_("    --csv                   emit output as CSV (not available for individual check output)\n"));
+	printf(_("    --nagios                emit output in Nagios format (individual check output only)\n"));
 	puts("");
 	printf(_("  Following options check an individual status:\n"));
 	printf(_("    --archive-ready         number of WAL files ready for archiving\n"));
@@ -2932,6 +3183,7 @@ do_node_help(void)
 	printf(_("    --role                  check node has expected role\n"));
 	printf(_("    --slots                 check for inactive replication slots\n"));
 	printf(_("    --missing-slots         check for missing replication slots\n"));
+	printf(_("    --data-directory-config check repmgr's data directory configuration\n"));
 
 	puts("");
 
@@ -2963,6 +3215,7 @@ do_node_help(void)
 	printf(_("    --dry-run             show what action would be performed, but don't execute it\n"));
 	printf(_("    --action              action to perform (one of \"start\", \"stop\", \"restart\" or \"reload\")\n"));
 	printf(_("    --list-actions        show what command would be performed for each action\n"));
+	printf(_("    --checkpoint          issue a CHECKPOINT before stopping or restarting the node\n"));
 	puts("");
 
 
