@@ -1,6 +1,6 @@
 /*
  * repmgr-client-global.h
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,14 +35,17 @@ typedef struct
 	bool		connection_param_provided;
 	bool		host_param_provided;
 	bool		limit_provided;
+	bool		wait_provided;
 
 	/* general configuration options */
 	char		config_file[MAXPGPATH];
 	bool		dry_run;
 	bool		force;
 	char		pg_bindir[MAXLEN];	/* overrides setting in repmgr.conf */
-	bool		wait;
+	int			wait;
 	bool		no_wait;
+	bool		compact;
+	bool		detail;
 
 	/* logging options */
 	char		log_level[MAXLEN];	/* overrides setting in repmgr.conf */
@@ -68,7 +71,7 @@ typedef struct
 
 	/* general node options */
 	int			node_id;
-	char		node_name[MAXLEN];
+	char		node_name[NAMEDATALEN];
 	char		data_dir[MAXPGPATH];
 	int			remote_node_id;
 
@@ -98,6 +101,7 @@ typedef struct
 	char		force_rewind_path[MAXPGPATH];
 	bool		siblings_follow;
 	bool		repmgrd_no_pause;
+	bool		repmgrd_force_unpause;
 
 	/* "node status" options */
 	bool		is_shutdown_cleanly;
@@ -111,8 +115,9 @@ typedef struct
 	bool		missing_slots;
 	bool		has_passfile;
 	bool		replication_connection;
+	bool		data_directory_config;
 
-	/* "node join" options */
+	/* "node rejoin" options */
 	char		config_files[MAXLEN];
 
 	/* "node service" options */
@@ -132,13 +137,15 @@ typedef struct
 	/* following options for internal use */
 	char		config_archive_dir[MAXPGPATH];
 	OutputMode	output_mode;
+	bool		disable_wal_receiver;
+	bool		enable_wal_receiver;
 } t_runtime_options;
 
 #define T_RUNTIME_OPTIONS_INITIALIZER { \
 		/* configuration metadata */ \
-		false, false, false, false,	\
+		false, false, false, false, false,	\
 		/* general configuration options */	\
-		"", false, false, "", false, false,	\
+		"", false, false, "", -1, false, false, false, \
 		/* logging options */ \
 		"", false, false, false, false,	\
 		/* output options */ \
@@ -157,12 +164,12 @@ typedef struct
 		/* "standby register" options */ \
 		false, -1, DEFAULT_WAIT_START,   \
 		/* "standby switchover" options */ \
-		false, false, "", false, false,	\
+		false, false, "", false, false, false,	\
 		/* "node status" options */ \
 		false, \
 		/* "node check" options */ \
-		false, false, false, false, false, false, false, false,	\
-		/* "node join" options */ \
+		false, false, false, false, false, false, false, false,	false, \
+		/* "node rejoin" options */ \
 		"", \
 		/* "node service" options */ \
 		"", false, false, false,  \
@@ -171,7 +178,7 @@ typedef struct
 		/* "cluster cleanup" options */ \
 		0, \
 		/* following options for internal use */ \
-		"/tmp", OM_TEXT	\
+		"/tmp", OM_TEXT, false, false \
 }
 
 
@@ -199,6 +206,7 @@ typedef struct ColHeader
 	char		title[MAXLEN];
 	int			max_length;
 	int			cur_length;
+	bool		display;
 } ColHeader;
 
 
@@ -220,8 +228,6 @@ extern int	check_server_version(PGconn *conn, char *server_type, bool exit_on_er
 extern void check_93_config(void);
 extern bool create_repmgr_extension(PGconn *conn);
 extern int	test_ssh_connection(char *host, char *remote_user);
-extern bool local_command(const char *command, PQExpBufferData *outputbuf);
-extern bool local_command_simple(const char *command, PQExpBufferData *outputbuf);
 
 extern standy_clone_mode get_standby_clone_mode(void);
 
@@ -234,11 +240,11 @@ extern char *make_pg_path(const char *file);
 
 extern void get_superuser_connection(PGconn **conn, PGconn **superuser_conn, PGconn **privileged_conn);
 
-extern bool remote_command(const char *host, const char *user, const char *command, PQExpBufferData *outputbuf);
-
 extern void make_remote_repmgr_path(PQExpBufferData *outputbuf, t_node_info *remote_node_record);
+extern void make_repmgrd_path(PQExpBufferData *output_buf);
 
 /* display functions */
+extern bool format_node_status(t_node_info *node_info, PQExpBufferData *node_status, PQExpBufferData *upstream, ItemList *warnings);
 extern void print_help_header(void);
 extern void print_status_header(int cols, ColHeader *headers);
 
@@ -251,4 +257,8 @@ extern void init_node_record(t_node_info *node_record);
 extern bool can_use_pg_rewind(PGconn *conn, const char *data_directory, PQExpBufferData *reason);
 extern void drop_replication_slot_if_exists(PGconn *conn, int node_id, char *slot_name);
 
+extern bool check_node_can_attach(TimeLineID local_tli, XLogRecPtr local_xlogpos, PGconn *follow_target_conn, t_node_info *follow_target_node_record, bool is_rejoin);
+extern void check_shared_library(PGconn *conn);
+extern bool is_repmgrd_running(PGconn *conn);
+
 #endif							/* _REPMGR_CLIENT_GLOBAL_H_ */
