@@ -1,6 +1,6 @@
 /*
  * repmgr-client.h
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,78 +40,92 @@
 #define NODE_CHECK			   14
 #define NODE_SERVICE		   15
 #define NODE_REJOIN            16
-#define CLUSTER_SHOW		   17
-#define CLUSTER_CLEANUP		   18
-#define CLUSTER_MATRIX		   19
-#define CLUSTER_CROSSCHECK	   20
-#define CLUSTER_EVENT		   21
-#define DAEMON_STATUS		   22
-#define DAEMON_PAUSE		   23
-#define DAEMON_UNPAUSE		   24
+#define NODE_CONTROL           17
+#define CLUSTER_SHOW		   18
+#define CLUSTER_CLEANUP		   19
+#define CLUSTER_MATRIX		   20
+#define CLUSTER_CROSSCHECK	   21
+#define CLUSTER_EVENT		   22
+#define DAEMON_STATUS		   23
+#define DAEMON_PAUSE		   24
+#define DAEMON_UNPAUSE		   25
+#define DAEMON_START 		   26
+#define DAEMON_STOP 		   27
 
 /* command line options without short versions */
 #define OPT_HELP						   1001
-#define OPT_CHECK_UPSTREAM_CONFIG		   1002
-#define OPT_COPY_EXTERNAL_CONFIG_FILES	   1003
-#define OPT_CSV							   1004
-#define OPT_NODE						   1005
-#define OPT_NODE_ID						   1006
-#define OPT_NODE_NAME					   1007
-#define OPT_WITHOUT_BARMAN				   1008
-#define OPT_NO_UPSTREAM_CONNECTION		   1009
-#define OPT_WAIT_SYNC				       1010
-#define OPT_LOG_TO_FILE					   1011
-#define OPT_UPSTREAM_CONNINFO			   1012
-#define OPT_REPLICATION_USER			   1013
-#define OPT_EVENT						   1014
-#define OPT_LIMIT						   1015
-#define OPT_ALL							   1016
-#define OPT_DRY_RUN						   1017
-#define OPT_UPSTREAM_NODE_ID			   1018
-#define OPT_ACTION						   1019
-#define OPT_LIST_ACTIONS				   1020
-#define OPT_CHECKPOINT					   1021
-#define OPT_IS_SHUTDOWN_CLEANLY			   1022
-#define OPT_ALWAYS_PROMOTE				   1023
-#define OPT_FORCE_REWIND				   1024
-#define OPT_NAGIOS						   1025
-#define OPT_ARCHIVE_READY				   1026
-#define OPT_OPTFORMAT					   1027
-#define OPT_REPLICATION_LAG				   1028
-#define OPT_CONFIG_FILES				   1029
-#define OPT_SIBLINGS_FOLLOW				   1030
-#define OPT_ROLE						   1031
-#define OPT_DOWNSTREAM					   1032
-#define OPT_SLOTS						   1033
-#define OPT_CONFIG_ARCHIVE_DIR			   1034
-#define OPT_HAS_PASSFILE                   1035
-#define OPT_WAIT_START                     1036
-#define OPT_REPL_CONN                      1037
-#define OPT_REMOTE_NODE_ID                 1038
-#define OPT_RECOVERY_CONF_ONLY             1039
-#define OPT_NO_WAIT                        1040
-#define OPT_MISSING_SLOTS                  1041
-#define OPT_REPMGRD_NO_PAUSE               1042
+#define OPT_COPY_EXTERNAL_CONFIG_FILES	   1002
+#define OPT_CSV							   1003
+#define OPT_NODE_ID						   1004
+#define OPT_NODE_NAME					   1005
+#define OPT_WITHOUT_BARMAN				   1006
+#define OPT_NO_UPSTREAM_CONNECTION		   1007
+#define OPT_WAIT_SYNC					   1008
+#define OPT_LOG_TO_FILE					   1009
+#define OPT_UPSTREAM_CONNINFO			   1010
+#define OPT_REPLICATION_USER			   1011
+#define OPT_EVENT						   1012
+#define OPT_LIMIT						   1013
+#define OPT_ALL							   1014
+#define OPT_DRY_RUN						   1015
+#define OPT_UPSTREAM_NODE_ID			   1016
+#define OPT_ACTION						   1017
+#define OPT_LIST_ACTIONS				   1018
+#define OPT_CHECKPOINT					   1019
+#define OPT_IS_SHUTDOWN_CLEANLY			   1020
+#define OPT_ALWAYS_PROMOTE				   1021
+#define OPT_FORCE_REWIND				   1022
+#define OPT_NAGIOS						   1023
+#define OPT_ARCHIVE_READY				   1024
+#define OPT_OPTFORMAT					   1025
+#define OPT_REPLICATION_LAG				   1026
+#define OPT_CONFIG_FILES				   1027
+#define OPT_SIBLINGS_FOLLOW				   1028
+#define OPT_ROLE						   1029
+#define OPT_DOWNSTREAM					   1030
+#define OPT_SLOTS						   1031
+#define OPT_CONFIG_ARCHIVE_DIR			   1032
+#define OPT_HAS_PASSFILE				   1033
+#define OPT_WAIT_START					   1034
+#define OPT_REPL_CONN					   1035
+#define OPT_REMOTE_NODE_ID				   1036
+#define OPT_RECOVERY_CONF_ONLY			   1037
+#define OPT_NO_WAIT						   1038
+#define OPT_MISSING_SLOTS				   1039
+#define OPT_REPMGRD_NO_PAUSE			   1040
+#define OPT_VERSION_NUMBER				   1041
+#define OPT_DATA_DIRECTORY_CONFIG		   1042
+#define OPT_COMPACT						   1043
+#define OPT_DISABLE_WAL_RECEIVER		   1044
+#define OPT_ENABLE_WAL_RECEIVER			   1045
+#define OPT_DETAIL						   1046
+#define OPT_REPMGRD_FORCE_UNPAUSE		   1047
 
 /* deprecated since 3.3 */
 #define OPT_DATA_DIR						999
 #define OPT_NO_CONNINFO_PASSWORD			998
 #define OPT_RECOVERY_MIN_APPLY_DELAY		997
+/* deprecated since 4.0 */
+#define OPT_CHECK_UPSTREAM_CONFIG		    996
+#define OPT_NODE						    995
 
 
 static struct option long_options[] =
 {
 /* general options */
-	{"version", no_argument, NULL, 'V'},
 	{"help", no_argument, NULL, OPT_HELP},
+	{"version", no_argument, NULL, 'V'},
+	{"version-number", no_argument, NULL, OPT_VERSION_NUMBER},
 
 /* general configuration options */
 	{"config-file", required_argument, NULL, 'f'},
 	{"dry-run", no_argument, NULL, OPT_DRY_RUN},
 	{"force", no_argument, NULL, 'F'},
 	{"pg_bindir", required_argument, NULL, 'b'},
-	{"wait", no_argument, NULL, 'w'},
+	{"wait", optional_argument, NULL, 'w'},
 	{"no-wait", no_argument, NULL, 'W'},
+	{"compact", no_argument, NULL, OPT_COMPACT},
+	{"detail", no_argument, NULL, OPT_DETAIL},
 
 /* connection options */
 	{"dbname", required_argument, NULL, 'd'},
@@ -156,11 +170,12 @@ static struct option long_options[] =
 
 /* "standby switchover" options
  *
- * Note: --force-rewind accepted to pass to "node join"
+ * Note: --force-rewind accepted to pass to "node rejoin"
  */
 	{"always-promote", no_argument, NULL, OPT_ALWAYS_PROMOTE},
 	{"siblings-follow", no_argument, NULL, OPT_SIBLINGS_FOLLOW},
 	{"repmgrd-no-pause", no_argument, NULL, OPT_REPMGRD_NO_PAUSE},
+	{"repmgrd-force-unpause", no_argument, NULL, OPT_REPMGRD_FORCE_UNPAUSE},
 
 /* "node status" options */
 	{"is-shutdown-cleanly", no_argument, NULL, OPT_IS_SHUTDOWN_CLEANLY},
@@ -174,6 +189,7 @@ static struct option long_options[] =
 	{"missing-slots", no_argument, NULL, OPT_MISSING_SLOTS},
 	{"has-passfile", no_argument, NULL, OPT_HAS_PASSFILE},
 	{"replication-connection", no_argument, NULL, OPT_REPL_CONN},
+	{"data-directory-config", no_argument, NULL, OPT_DATA_DIRECTORY_CONFIG},
 
 /* "node rejoin" options */
 	{"config-files", required_argument, NULL, OPT_CONFIG_FILES},
@@ -193,6 +209,10 @@ static struct option long_options[] =
 /* "cluster cleanup" options */
 	{"keep-history", required_argument, NULL, 'k'},
 
+/* undocumented options for testing */
+	{"disable-wal-receiver", no_argument, NULL, OPT_DISABLE_WAL_RECEIVER},
+	{"enable-wal-receiver", no_argument, NULL, OPT_ENABLE_WAL_RECEIVER},
+
 /* deprecated */
 	{"check-upstream-config", no_argument, NULL, OPT_CHECK_UPSTREAM_CONFIG},
 	{"no-conninfo-password", no_argument, NULL, OPT_NO_CONNINFO_PASSWORD},
