@@ -1,7 +1,7 @@
 /*
  * repmgr.c - repmgr extension
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This is the actual extension code; see repmgr-client.c for the code which
  * generates the repmgr binary
@@ -40,7 +40,6 @@
 
 #include "utils/timestamp.h"
 
-#include "executor/spi.h"
 #include "lib/stringinfo.h"
 #include "access/xact.h"
 #include "utils/snapmgr.h"
@@ -54,11 +53,12 @@
 #include "voting.h"
 
 #define UNKNOWN_NODE_ID		-1
+#define ELECTION_RERUN_NOTIFICATION -2
 #define UNKNOWN_PID			-1
 
 #define TRANCHE_NAME "repmgrd"
 #define REPMGRD_STATE_FILE PGSTAT_STAT_PERMANENT_DIRECTORY "/repmgrd_state.txt"
-
+#define REPMGRD_STATE_FILE_BUF_SIZE 128
 
 PG_MODULE_MAGIC;
 
@@ -78,6 +78,8 @@ typedef struct repmgrdSharedState
 	char		repmgrd_pidfile[MAXPGPATH];
 	bool		repmgrd_paused;
 	/* streaming failover */
+	int			upstream_node_id;
+	TimestampTz upstream_last_seen;
 	NodeVotingStatus voting_status;
 	int			current_electoral_term;
 	int			candidate_node_id;
@@ -108,6 +110,18 @@ PG_FUNCTION_INFO_V1(standby_set_last_updated);
 Datum		standby_get_last_updated(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(standby_get_last_updated);
 
+Datum		set_upstream_last_seen(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(set_upstream_last_seen);
+
+Datum		get_upstream_last_seen(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(get_upstream_last_seen);
+
+Datum		get_upstream_node_id(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(get_upstream_node_id);
+
+Datum		set_upstream_node_id(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(set_upstream_node_id);
+
 Datum		notify_follow_primary(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(notify_follow_primary);
 
@@ -141,6 +155,8 @@ PG_FUNCTION_INFO_V1(repmgrd_pause);
 Datum		repmgrd_is_paused(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(repmgrd_is_paused);
 
+Datum		get_wal_receiver_pid(PG_FUNCTION_ARGS);
+PG_FUNCTION_INFO_V1(get_wal_receiver_pid);
 
 
 /*
@@ -219,6 +235,9 @@ repmgr_shmem_startup(void)
 		memset(shared_state->repmgrd_pidfile, 0, MAXPGPATH);
 		shared_state->repmgrd_paused = false;
 		shared_state->current_electoral_term = 0;
+		shared_state->upstream_node_id = UNKNOWN_NODE_ID;
+		/* arbitrary "magic" date to indicate this field hasn't been updated */
+		shared_state->upstream_last_seen = POSTGRES_EPOCH_JDATE;
 		shared_state->voting_status = VS_NO_VOTE;
 		shared_state->candidate_node_id = UNKNOWN_NODE_ID;
 		shared_state->follow_new_primary = false;
@@ -256,8 +275,8 @@ set_local_node_id(PG_FUNCTION_ARGS)
 
 		if (file != NULL)
 		{
-			int			buffer_size = 128;
-			char		buffer[buffer_size];
+			int			buffer_size = REPMGRD_STATE_FILE_BUF_SIZE;
+			char		buffer[REPMGRD_STATE_FILE_BUF_SIZE];
 
 			if (fgets(buffer, buffer_size, file) != NULL)
 			{
@@ -354,6 +373,107 @@ standby_get_last_updated(PG_FUNCTION_ARGS)
 }
 
 
+Datum
+set_upstream_last_seen(PG_FUNCTION_ARGS)
+{
+	int			upstream_node_id = UNKNOWN_NODE_ID;
+
+	if (!shared_state)
+		PG_RETURN_VOID();
+
+	if (PG_ARGISNULL(0))
+		PG_RETURN_NULL();
+
+	upstream_node_id = PG_GETARG_INT32(0);
+
+	LWLockAcquire(shared_state->lock, LW_EXCLUSIVE);
+
+	shared_state->upstream_last_seen = GetCurrentTimestamp();
+	shared_state->upstream_node_id = upstream_node_id;
+	LWLockRelease(shared_state->lock);
+
+	PG_RETURN_VOID();
+}
+
+
+Datum
+get_upstream_last_seen(PG_FUNCTION_ARGS)
+{
+	long		secs;
+	int			microsecs;
+	TimestampTz last_seen;
+
+	if (!shared_state)
+		PG_RETURN_INT32(-1);
+
+	LWLockAcquire(shared_state->lock, LW_SHARED);
+
+	last_seen = shared_state->upstream_last_seen;
+
+	LWLockRelease(shared_state->lock);
+
+	/*
+	 * "last_seen" is initialised with the PostgreSQL epoch as a
+	 * "magic" value to indicate the field hasn't ever been updated
+	 * by repmgrd. We return -1 instead, rather than imply that the
+	 * primary was last seen at the turn of the century.
+	 */
+	if (last_seen == POSTGRES_EPOCH_JDATE)
+		PG_RETURN_INT32(-1);
+
+
+	TimestampDifference(last_seen, GetCurrentTimestamp(),
+						&secs, &microsecs);
+
+	/* let's hope repmgrd never runs for more than a century or so without seeing a primary */
+	PG_RETURN_INT32((uint32)secs);
+}
+
+
+Datum
+get_upstream_node_id(PG_FUNCTION_ARGS)
+{
+	int			upstream_node_id = UNKNOWN_NODE_ID;
+
+	if (!shared_state)
+		PG_RETURN_NULL();
+
+	LWLockAcquire(shared_state->lock, LW_SHARED);
+	upstream_node_id = shared_state->upstream_node_id;
+	LWLockRelease(shared_state->lock);
+
+	PG_RETURN_INT32(upstream_node_id);
+}
+
+Datum
+set_upstream_node_id(PG_FUNCTION_ARGS)
+{
+	int			upstream_node_id = UNKNOWN_NODE_ID;
+	int			local_node_id = UNKNOWN_NODE_ID;
+
+	if (!shared_state)
+		PG_RETURN_NULL();
+
+	if (PG_ARGISNULL(0))
+		PG_RETURN_NULL();
+
+	upstream_node_id = PG_GETARG_INT32(0);
+
+	LWLockAcquire(shared_state->lock, LW_SHARED);
+	local_node_id = shared_state->local_node_id;
+	LWLockRelease(shared_state->lock);
+
+	if (local_node_id == upstream_node_id)
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+				 (errmsg("upstream node id cannot be the same as the local node id"))));
+
+	LWLockAcquire(shared_state->lock, LW_EXCLUSIVE);
+	shared_state->upstream_node_id = upstream_node_id;
+	LWLockRelease(shared_state->lock);
+
+	PG_RETURN_VOID();
+}
 
 
 /* ===================*/
@@ -367,10 +487,10 @@ notify_follow_primary(PG_FUNCTION_ARGS)
 	int			primary_node_id = UNKNOWN_NODE_ID;
 
 	if (!shared_state)
-		PG_RETURN_NULL();
+		PG_RETURN_VOID();
 
 	if (PG_ARGISNULL(0))
-		PG_RETURN_NULL();
+		PG_RETURN_VOID();
 
 	primary_node_id = PG_GETARG_INT32(0);
 
@@ -379,9 +499,17 @@ notify_follow_primary(PG_FUNCTION_ARGS)
 	/* only do something if local_node_id is initialised */
 	if (shared_state->local_node_id != UNKNOWN_NODE_ID)
 	{
-		elog(INFO, "node %i received notification to follow node %i",
-			 shared_state->local_node_id,
-			 primary_node_id);
+		if (primary_node_id == ELECTION_RERUN_NOTIFICATION)
+		{
+			elog(INFO, "node %i received notification to rerun promotion candidate election",
+				 shared_state->local_node_id);
+		}
+		else
+		{
+			elog(INFO, "node %i received notification to follow node %i",
+				 shared_state->local_node_id,
+				 primary_node_id);
+		}
 
 		LWLockRelease(shared_state->lock);
 		LWLockAcquire(shared_state->lock, LW_EXCLUSIVE);
@@ -402,7 +530,7 @@ get_new_primary(PG_FUNCTION_ARGS)
 	int			new_primary_node_id = UNKNOWN_NODE_ID;
 
 	if (!shared_state)
-		PG_RETURN_NULL();
+		PG_RETURN_INT32(UNKNOWN_NODE_ID);
 
 	LWLockAcquire(shared_state->lock, LW_SHARED);
 
@@ -412,7 +540,7 @@ get_new_primary(PG_FUNCTION_ARGS)
 	LWLockRelease(shared_state->lock);
 
 	if (new_primary_node_id == UNKNOWN_NODE_ID)
-		PG_RETURN_NULL();
+		PG_RETURN_INT32(UNKNOWN_NODE_ID);
 
 	PG_RETURN_INT32(new_primary_node_id);
 }
@@ -638,9 +766,8 @@ repmgrd_pause(PG_FUNCTION_ARGS)
 
 	if (file == NULL)
 	{
-		elog(DEBUG1, "unable to allocate %s", REPMGRD_STATE_FILE);
+		elog(WARNING, "unable to allocate %s", REPMGRD_STATE_FILE);
 
-		// XXX anything else we can do? log?
 		PG_RETURN_VOID();
 	}
 
@@ -655,11 +782,13 @@ repmgrd_pause(PG_FUNCTION_ARGS)
 					 pause ? 1 : 0);
 	LWLockRelease(shared_state->lock);
 
-	// XXX check success
-	fwrite(buf.data, strlen(buf.data) + 1, 1, file);
+	if (fwrite(buf.data, strlen(buf.data) + 1, 1, file) != 1)
+	{
+		elog(WARNING, _("unable to write to file %s"), REPMGRD_STATE_FILE);
+	}
 
+	pfree(buf.data);
 
-	resetStringInfo(&buf);
 	FreeFile(file);
 
 	PG_RETURN_VOID();
@@ -680,3 +809,17 @@ repmgrd_is_paused(PG_FUNCTION_ARGS)
 
 	PG_RETURN_BOOL(is_paused);
 }
+
+
+Datum
+get_wal_receiver_pid(PG_FUNCTION_ARGS)
+{
+	int wal_receiver_pid;
+
+	if (!shared_state)
+		PG_RETURN_NULL();
+
+	wal_receiver_pid = WalRcv->pid;
+
+	PG_RETURN_INT32(wal_receiver_pid);
+}
