@@ -1,7 +1,7 @@
 /*
  * repmgrd.c - Replication manager daemon
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -88,6 +88,7 @@ main(int argc, char **argv)
 
 	RecordStatus record_status;
 	ExtensionStatus extension_status = REPMGR_UNKNOWN;
+	t_extension_versions extversions = T_EXTENSION_VERSIONS_INITIALIZER;
 
 	FILE	   *fd;
 
@@ -371,12 +372,6 @@ main(int argc, char **argv)
 	/* abort if local node not available at startup */
 	local_conn = establish_db_connection(config_file_options.conninfo, true);
 
-	/*
-	 * store the server version number - we'll need this to generate
-	 * version-dependent queries etc.
-	 */
-	server_version_num = get_server_version(local_conn, NULL);
-
 	/*
 	 * sanity checks
 	 *
@@ -388,33 +383,86 @@ main(int argc, char **argv)
 	 * repmgr has not been properly configured.
 	 */
 
-	/* Check "repmgr" the extension is installed */
-	extension_status = get_repmgr_extension_status(local_conn);
 
-	if (extension_status != REPMGR_INSTALLED)
+	/* warn about any settings which might not be relevant for the current PostgreSQL version  */
+	if (config_file_options.standby_disconnect_on_failover == true && PQserverVersion(local_conn) < 90500)
+	{
+		log_warning(_("\"standby_disconnect_on_failover\" specified, but not available for this PostgreSQL version"));
+		/* TODO: format server version */
+		log_detail(_("available from PostgreSQL 9.5, this PostgreSQL version is %i"), PQserverVersion(local_conn));
+	}
+
+	/* Check "repmgr" the extension is installed */
+	extension_status = get_repmgr_extension_status(local_conn, &extversions);
+
+	if (extension_status == REPMGR_INSTALLED)
+	{
+		/*
+		 * extension is the latest available according to "pg_available_extensions" -
+		 * - does our (major) version match that?
+		 */
+		log_verbose(LOG_DEBUG, "binary version: %i; extension version: %i",
+					REPMGR_VERSION_NUM, extversions.installed_version_num);
+		if ((REPMGR_VERSION_NUM/100) < (extversions.installed_version_num / 100))
+		{
+			log_error(_("this \"repmgr\" version is older than the installed \"repmgr\" extension version"));
+			log_detail(_("\"repmgr\" version %s is installed but extension is version %s"),
+					   REPMGR_VERSION,
+					   extversions.installed_version);
+			log_hint(_("update the repmgr binaries to match the installed extension version"));
+
+			close_connection(&local_conn);
+			exit(ERR_BAD_CONFIG);
+		}
+
+		if ((REPMGR_VERSION_NUM/100) > (extversions.installed_version_num / 100))
+		{
+			log_error(_("this \"repmgr\" version is newer than the installed \"repmgr\" extension version"));
+			log_detail(_("\"repmgr\" version %s is installed but extension is version %s"),
+					   REPMGR_VERSION,
+					   extversions.installed_version);
+			log_hint(_("update the installed extension version by executing \"ALTER EXTENSION repmgr UPDATE\""));
+
+			close_connection(&local_conn);
+			exit(ERR_BAD_CONFIG);
+		}
+	}
+	else
 	{
 		/* this is unlikely to happen */
 		if (extension_status == REPMGR_UNKNOWN)
 		{
 			log_error(_("unable to determine status of \"repmgr\" extension"));
-			log_detail("%s", PQerrorMessage(local_conn));
+			log_detail("\n%s", PQerrorMessage(local_conn));
 			close_connection(&local_conn);
 			exit(ERR_DB_QUERY);
 		}
 
-		log_error(_("repmgr extension not found on this node"));
-
-		if (extension_status == REPMGR_AVAILABLE)
+		if (extension_status == REPMGR_OLD_VERSION_INSTALLED)
 		{
-			log_detail(_("repmgr extension is available but not installed in database \"%s\""),
-					   PQdb(local_conn));
+			log_error(_("an older version of the \"repmgr\" extension is installed"));
+			log_detail(_("extension version %s is installed but newer version %s is available"),
+					   extversions.installed_version,
+					   extversions.default_version);
+			log_hint(_("verify the repmgr installation is updated properly before continuing"));
 		}
-		else if (extension_status == REPMGR_UNAVAILABLE)
+		else
 		{
-			log_detail(_("repmgr extension is not available on this node"));
+			log_error(_("repmgr extension not found on this node"));
+
+			if (extension_status == REPMGR_AVAILABLE)
+			{
+				log_detail(_("repmgr extension is available but not installed in database \"%s\""),
+						   PQdb(local_conn));
+			}
+			else if (extension_status == REPMGR_UNAVAILABLE)
+			{
+				log_detail(_("repmgr extension is not available on this node"));
+			}
+
+			log_hint(_("check that this node is part of a repmgr cluster"));
 		}
 
-		log_hint(_("check that this node is part of a repmgr cluster"));
 		close_connection(&local_conn);
 		exit(ERR_BAD_CONFIG);
 	}
@@ -513,6 +561,8 @@ start_monitoring(void)
 			   local_node_info.node_name,
 			   local_node_info.node_id);
 
+	log_info(_("\"connection_check_type\" set to \"%s\""), print_connection_check_type(config_file_options.connection_check_type));
+
 	while (true)
 	{
 		switch (local_node_info.type)
@@ -579,7 +629,8 @@ daemonize_process(void)
 	switch (pid)
 	{
 		case -1:
-			log_error(_("error in fork():\n  %s"), strerror(errno));
+			log_error(_("error in fork()"));
+			log_detail("%s", strerror(errno));
 			exit(ERR_SYS_FAILURE);
 			break;
 
@@ -588,7 +639,8 @@ daemonize_process(void)
 			pid = setsid();
 			if (pid == (pid_t) -1)
 			{
-				log_error(_("error in setsid():\n  %s"), strerror(errno));
+				log_error(_("error executing setsid()"));
+				log_detail("%s", strerror(errno));
 				exit(ERR_SYS_FAILURE);
 			}
 
@@ -598,7 +650,8 @@ daemonize_process(void)
 			/* error case */
 			if (pid == -1)
 			{
-				log_error(_("error in fork():\n  %s"), strerror(errno));
+				log_error(_("error executing fork()"));
+				log_detail("%s", strerror(errno));
 				exit(ERR_SYS_FAILURE);
 			}
 
@@ -776,6 +829,82 @@ show_help(void)
 }
 
 
+bool
+check_upstream_connection(PGconn **conn, const char *conninfo)
+{
+	/* Check the connection status twice in case it changes after reset */
+	bool		twice = false;
+
+	if (config_file_options.connection_check_type == CHECK_PING)
+		return is_server_available(conninfo);
+
+	if (config_file_options.connection_check_type == CHECK_CONNECTION)
+	{
+		bool success = true;
+		PGconn *test_conn = PQconnectdb(conninfo);
+
+		log_debug("check_upstream_connection(): attempting to connect to \"%s\"", conninfo);
+
+		if (PQstatus(test_conn) != CONNECTION_OK)
+		{
+			log_warning(_("unable to connect to \"%s\""), conninfo);
+			log_detail("\n%s", PQerrorMessage(test_conn));
+			success = false;
+		}
+		PQfinish(test_conn);
+
+		return success;
+	}
+
+	for (;;)
+	{
+		if (PQstatus(*conn) != CONNECTION_OK)
+		{
+			log_debug("check_upstream_connection(): connection not OK");
+			if (twice)
+				return false;
+			/* reconnect */
+			PQfinish(*conn);
+			*conn = PQconnectdb(conninfo);
+			twice = true;
+		}
+		else
+		{
+			if (!cancel_query(*conn, config_file_options.async_query_timeout))
+				goto failed;
+
+			if (wait_connection_availability(*conn, config_file_options.async_query_timeout) != 1)
+				goto failed;
+
+			/* execute a simple query to verify connection availability */
+			if (PQsendQuery(*conn, "SELECT 1") == 0)
+			{
+				log_warning(_("unable to send query to upstream"));
+				log_detail("%s", PQerrorMessage(*conn));
+				goto failed;
+			}
+
+			if (wait_connection_availability(*conn, config_file_options.async_query_timeout) != 1)
+				goto failed;
+
+			break;
+
+	failed:
+			/* retry once */
+			if (twice)
+				return false;
+
+			/* reconnect */
+			PQfinish(*conn);
+			*conn = PQconnectdb(conninfo);
+			twice = true;
+		}
+	}
+
+	return true;
+}
+
+
 void
 try_reconnect(PGconn **conn, t_node_info *node_info)
 {
@@ -801,15 +930,13 @@ try_reconnect(PGconn **conn, t_node_info *node_info)
 				 node_info->node_id, i + 1, max_attempts);
 		if (is_server_available_params(&conninfo_params) == true)
 		{
-
-			log_notice(_("node has recovered, reconnecting"));
+			log_notice(_("node %i has recovered, reconnecting"), node_info->node_id);
 
 			/*
-			 * XXX we should also handle the case where node is pingable but
-			 * connection denied due to connection exhaustion - fall back to
-			 * degraded monitoring? - make that configurable
+			 * Note: we could also handle the case where node is pingable but
+			 * connection denied due to connection exhaustion, by falling back to
+			 * degraded monitoring (make configurable)
 			 */
-
 			our_conn = establish_db_connection_by_params(&conninfo_params, false);
 
 			if (PQstatus(our_conn) == CONNECTION_OK)
@@ -820,7 +947,7 @@ try_reconnect(PGconn **conn, t_node_info *node_info)
 
 				if (PQstatus(*conn) == CONNECTION_BAD)
 				{
-					log_verbose(LOG_INFO, "original connection handle returned CONNECTION_BAD, using new connection");
+					log_verbose(LOG_INFO, _("original connection handle returned CONNECTION_BAD, using new connection"));
 					close_connection(conn);
 					*conn = our_conn;
 				}
@@ -832,7 +959,7 @@ try_reconnect(PGconn **conn, t_node_info *node_info)
 
 					if (ping_result != PGRES_TUPLES_OK)
 					{
-						log_info("original connnection no longer available, using new connection");
+						log_info(_("original connection no longer available, using new connection"));
 						close_connection(conn);
 						*conn = our_conn;
 					}
@@ -850,7 +977,9 @@ try_reconnect(PGconn **conn, t_node_info *node_info)
 			}
 
 			close_connection(&our_conn);
-			log_notice(_("unable to reconnect to node %i"), node_info->node_id);
+			log_notice(_("unable to reconnect to node \"%s\" (ID: %i)"),
+					   node_info->node_name,
+					   node_info->node_id);
 		}
 
 		if (i + 1 < max_attempts)
