@@ -1,7 +1,7 @@
 /*
  * strutil.c
  *
- * Copyright (c) 2ndQuadrant, 2010-2018
+ * Copyright (c) 2ndQuadrant, 2010-2019
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -115,6 +115,9 @@ item_list_append_format(ItemList *item_list, const char *format,...)
 	ItemListCell *cell;
 	va_list		arglist;
 
+	if (item_list == NULL)
+		return;
+
 	cell = (ItemListCell *) pg_malloc0(sizeof(ItemListCell));
 
 	if (cell == NULL)
